import com.rti.dds.domain.DomainParticipant;
import com.rti.dds.domain.DomainParticipantFactory;

import com.rti.dds.topic.Topic;

import com.rti.dds.infrastructure.*;

import com.rti.dds.type.builtin.StringTypeSupport;
import com.rti.dds.type.builtin.StringDataWriter;

import com.rti.dds.publication.*;

public class TestPublisher
{
  public static final void main(String[] args)
  {
    // Create the DDS DomainParticipant
    DomainParticipant domain =
      DomainParticipantFactory.get_instance().create_participant(
          0,                                                // Domain ID
          DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT, // Use default QoS
          null,                                             // No domain lstnr
          StatusKind.STATUS_MASK_NONE);

    if (domain == null)
    {
      System.err.println("Cannot create domain");
      return;
    }

    // Create a topic that the Subscriber can register for
    Topic topic = domain.create_topic(
        "Test App Topic",                    // Topic name. Sub uses same name
        StringTypeSupport.get_type_name(),   // We'll pass strings
        DomainParticipant.TOPIC_QOS_DEFAULT, // Use default QoS
        null,                                // No topic issue listener
        StatusKind.STATUS_MASK_NONE);        // No listener so N/A

    if (topic == null)
    {
      System.err.println("Cannot create topic");
      return;
    }

    // Make message sending reliable
    DataWriterQos writerQOS = new DataWriterQos();
    domain.get_default_datawriter_qos(writerQOS);

    writerQOS.reliability.kind =
      ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
    writerQOS.history.kind = HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;

    // Create the DataWriter
    StringDataWriter dataWriter =
      (StringDataWriter) domain.create_datawriter(
          topic,
          writerQOS,
          null,
          StatusKind.STATUS_MASK_NONE);

    if (dataWriter == null)
    {
      System.err.println("Cannot create Data Writer");
      return;
    }

    // Send a bunch of messages quickly
    for (int count=0; count<501; count++)
    {
/* <-- All messages are received if this comment is removed.
      // Pause every 32 sends (32 = receive buffer size?)
      if (count % 32 == 31)
      {
        try { Thread.sleep(100); } catch (Exception e) {}
      }
*/
      String msg = "This is message #" + count;
      dataWriter.write(msg, InstanceHandle_t.HANDLE_NIL);
    }

    // Wait for messages to be received
    System.out.println("Waiting for ackn...");
    Duration_t wait = new Duration_t();
    wait.sec = Duration_t.DURATION_INFINITE_SEC;
    wait.nanosec = Duration_t.DURATION_INFINITE_SEC;
    dataWriter.wait_for_acknowledgments(wait);

    // Graceful close
    System.out.println("Exiting...");
    domain.delete_contained_entities();
    DomainParticipantFactory.get_instance().delete_participant(domain);
  }
}
