import java.util.Date;

import com.rti.dds.subscription.DataReader;
import com.rti.dds.subscription.DataReaderAdapter;
import com.rti.dds.subscription.*;

import com.rti.dds.domain.DomainParticipant;
import com.rti.dds.domain.DomainParticipantFactory;

import com.rti.dds.infrastructure.*;

import com.rti.dds.topic.Topic;

import com.rti.dds.type.builtin.StringTypeSupport;
import com.rti.dds.type.builtin.StringDataReader;

public class TestSubscriber extends DataReaderAdapter
{
  private static int count;
  private static Object mutex;

  public static final void main(String[] args)
  {
    TestSubscriber.count = 0;
    TestSubscriber.mutex = new Object();

    // The Publisher works in Domain 0, so so should we
    DomainParticipant domain =
      DomainParticipantFactory.get_instance().create_participant(
          0,
          DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
          null,
          StatusKind.STATUS_MASK_NONE);

    if (domain == null)
    {
      System.err.println("Cannot join domain 0");
      return;
    }

    // We listen for messages on the topic named "Test App Topic"
    Topic topic = domain.create_topic(
        "Test App Topic",
        StringTypeSupport.get_type_name(),    // We'll pass strings
        DomainParticipant.TOPIC_QOS_DEFAULT,
        null,
        StatusKind.STATUS_MASK_NONE);

    if (topic == null)
    {
      System.err.println("Cannot create topic");
      return;
    }

    // We want a reliable network connection (no messages lost)
    DataReaderQos readerQOS = new DataReaderQos();
    domain.get_default_datareader_qos(readerQOS);

    readerQOS.reliability.kind =
      ReliabilityQosPolicyKind.RELIABLE_RELIABILITY_QOS;
    readerQOS.history.kind = HistoryQosPolicyKind.KEEP_ALL_HISTORY_QOS;

    // Messages are read by a DDS Data Reader. Ours must read strings.
    StringDataReader dataReader =
      (StringDataReader) domain.create_datareader(
          topic,
          readerQOS,
          new TestSubscriber(),    // Make another instance of ourselves to
                                   // accept incoming messages
          StatusKind.DATA_AVAILABLE_STATUS);
                                              
    if (dataReader == null)
    {
      System.out.println("Cannot create Data Reader");
      return;
    }

    // Wait for all messages to be read
    System.out.println("Waiting for messages...");
    int lastCount = -1;
    Date lastCountChange = new Date();
    while(true)
    {
      // Check 10 times a second
      try { Thread.sleep(100); }
      catch(InterruptedException e) {}

      synchronized (TestSubscriber.mutex)
      {
        // As long as the count is changing, messages are being received
        if (lastCount != TestSubscriber.count)
        {
          lastCount = TestSubscriber.count;
          lastCountChange = new Date();
        }

        // If no message has been received for a while, quit
        else
        {
          Date now = new Date();
          if ((now.getTime() - lastCountChange.getTime() > 4000) // 4 seconds
              && TestSubscriber.count > 0)
            break;
        }
      }
    }

    System.out.println("Received " + TestSubscriber.count + " messages");
    System.out.println("Shutting down...");
    domain.delete_contained_entities();
    DomainParticipantFactory.get_instance().delete_participant(domain);
  }

  // Called when a message is received. Just count the messages.
  public void on_data_available(DataReader reader)
  {
    StringDataReader stringReader = (StringDataReader)reader;
    SampleInfo info = new SampleInfo();
    String msg = null;
    while(true)
    {
      try
      {
        msg = stringReader.take_next_sample(info);
      }
      catch (RETCODE_NO_DATA noData) { break; }
      catch (RETCODE_ERROR e) { e.printStackTrace(); }

      if (info.valid_data)
      {
        System.out.println("Received: \"" + msg + "\"");
        synchronized(TestSubscriber.mutex)
        {
          TestSubscriber.count++;
        }
      }
    }
  }
}
